/*-------------------------------------------------------------------------
 *
 * UUID generation functions for FreeBSD
 *
 * Largely stolen by Andrew Gierth from contrib/uuid-ossp, which is
 * Copyright (c) 2007-2008 PostgreSQL Global Development Group
 *
 *-------------------------------------------------------------------------
 */

#include "postgres.h"
#include "fmgr.h"
#include "utils/builtins.h"
#include "utils/uuid.h"

/* OS has a uuid_hash that conflicts with ours; kill it*/
#define uuid_hash freebsd_uuid_hash

/* explicit path since we do _not_ want to get any other version */
#include "/usr/include/uuid.h"

PG_MODULE_MAGIC;

Datum uuid_nil(PG_FUNCTION_ARGS);
Datum uuid_generate_v1(PG_FUNCTION_ARGS);

PG_FUNCTION_INFO_V1(uuid_nil);
PG_FUNCTION_INFO_V1(uuid_generate_v1);

/* we assume that the string representation is portable and that the
 * native binary representation might not be. */

static Datum
internal_uuid_create(int v)
{
    Datum res;
    char *str = NULL;
    uint32_t status;
    if (v)
    {
	uuid_t uu;
	uuid_create(&uu, &status);
	if (status == uuid_s_ok)
	    uuid_to_string(&uu, &str, &status);
    }
    else
	uuid_to_string(NULL, &str, &status);

    if (status != uuid_s_ok)
    {
	if (str)
	    free(str);
	ereport(ERROR,
		(errcode(ERRCODE_EXTERNAL_ROUTINE_EXCEPTION),
		 errmsg("FreeBSD uuid library failure: %d", (int) status)));
    }

    res = DirectFunctionCall1(uuid_in, CStringGetDatum(str));

    free(str);

    return res;
}

Datum
uuid_nil(PG_FUNCTION_ARGS)
{
    return internal_uuid_create(0);
}

Datum
uuid_generate_v1(PG_FUNCTION_ARGS)
{
    return internal_uuid_create(1);
}

